/* notes.h generated by valac 0.56.18, the Vala compiler, do not modify */

#ifndef __SRC_NOTES_H__
#define __SRC_NOTES_H__

#include <libxfce4panel/libxfce4panel.h>
#include <glib-object.h>
#include <glib.h>

G_BEGIN_DECLS

#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define TYPE_NOTES_PLUGIN (notes_plugin_get_type ())
#define NOTES_PLUGIN(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_NOTES_PLUGIN, NotesPlugin))
#define NOTES_PLUGIN_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_NOTES_PLUGIN, NotesPluginClass))
#define IS_NOTES_PLUGIN(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_NOTES_PLUGIN))
#define IS_NOTES_PLUGIN_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_NOTES_PLUGIN))
#define NOTES_PLUGIN_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_NOTES_PLUGIN, NotesPluginClass))

typedef struct _NotesPlugin NotesPlugin;
typedef struct _NotesPluginClass NotesPluginClass;
typedef struct _NotesPluginPrivate NotesPluginPrivate;

struct _NotesPlugin {
	XfcePanelPlugin parent_instance;
	NotesPluginPrivate * priv;
};

struct _NotesPluginClass {
	XfcePanelPluginClass parent_class;
};

GType notes_plugin_register_type (GTypeModule * module);
VALA_EXTERN GType notes_plugin_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (NotesPlugin, g_object_unref)
VALA_EXTERN NotesPlugin* notes_plugin_new (void);
VALA_EXTERN NotesPlugin* notes_plugin_construct (GType object_type);
VALA_EXTERN GType xfce_panel_module_init (GTypeModule* module);

G_END_DECLS

#endif
